import pandas as pd

from dhlab.api.dhlab_api import get_dispersion


class Dispersion:
    def __init__(self, urn=None, wordbag=None, window=1000, pr=100):
        if isinstance(wordbag, list):
            dispersion = {w: get_dispersion(urn, words=[w], window=window, pr=pr) for w in wordbag}
        elif isinstance(wordbag, dict):
            dispersion = {w: get_dispersion(urn, words=wordbag[w], window=window, pr=pr) for w in
                          wordbag}
        elif isinstance(wordbag, str):
            dispersion = {wordbag: get_dispersion(urn, words=[wordbag], window=window, pr=pr)}
        else:
            dispersion = {}
        self.dispersion = pd.DataFrame(dispersion)

    def plot(self, **kwargs):
        self.dispersion.plot(**kwargs)



import contextlib
import os
from pathlib import Path

import requests
from IPython.display import Markdown, display


def printmd(S):
    """Use ``IPython.display`` to render markdown text."""
    display(Markdown(S))
    return


def download_from_github(filename=None, user=None, repository=None, branch='master',
                         overwrite=False,
                         silent=False):
    """Fetch a file from Github and write it to working directory.

    :param filename:     Filename, including file extension (e.g. `.py` or `.txt`)
    :param user:         Github username of the repo owner.
    :param repository:   Github repository name.
    :param branch:       Name of the repo branch. Defaults to 'master'.
    :param overwrite:    Whether to overwrite existing files in working directory. Defaults to not
                         overwrite.
    :param silent:       Whether to output logging messages to stdout. Default is not silent.
    """

    nba = requests.get(
        f"https://raw.githubusercontent.com/{user}/{repository}/{branch}/{filename}",
        headers={'Cache-Control': 'no-cache'}
    )
    if nba.status_code == 200:
        file_exists = os.path.exists(filename)
        if file_exists and not overwrite:
            if not silent:
                printmd(
                    f"File {os.path.abspath(filename)} exists - call `download_from_github('{filename}', overwrite "
                    f"= True)` in order to download module `{filename}` anyway")
        else:
            with open(filename, 'w+', encoding='utf-8') as pyfile:
                pyfile.write(nba.text)
                pyfile.flush()
                pyfile.close()
            if not silent:
                printmd(f"Downloaded file `{os.path.abspath(filename)}`")
    else:
        printmd(f"Failed to download {filename} with http code {nba.status_code}")


def get_file_from_github(url, overwrite=False, silent=False):
    """Fetch a file on github.

    it is enough with reference
    it will look in raw user content for the file.
    
    :param overwrite: defaults to no overwrite
    :param silent: default is not silent"""

    if url.startswith("https://github.com/") or url.startswith("github.com"):
        trail = url.split("github.com")[-1].replace('blob/', '')
        fileref = f"""https://raw.githubusercontent.com{trail}"""
    elif url.startswith("raw.githubusercontent"):
        fileref = f"""https://{url}"""
    elif url.startswith("https://raw.githubusercontent"):
        fileref = url
    else:
        return "is this is a file on github?"

    nba = requests.get(
        f"{fileref}",
        headers={'Cache-Control': 'no-cache'}
    )
    if nba.status_code == 200:
        filename = os.path.basename(fileref)
        file_exists = os.path.exists(filename)
        if file_exists and not overwrite:
            if not silent:
                printmd(
                    f"File {os.path.abspath(filename)} exists -"
                    f" call `download('{filename}', overwrite = True)` in order to "
                    f"download `{filename}` anyway")
        else:
            nba = nba.text
            with open(filename, 'w', encoding='UTF-8') as pyfile:
                pyfile.write(nba)
                pyfile.flush()
                pyfile.close()
            if not silent:
                printmd(f"Downloaded file `{os.path.abspath(filename)}`")
    else:
        printmd(f"Failed to download {fileref} with http response code {nba.status_code}")

    return


@contextlib.contextmanager
def working_directory(path):
    """Changes working directory and returns to previous on exit.

    Source: https://stackoverflow.com/a/42441759

    :meta private:
    """
    prev_cwd = Path.cwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(prev_cwd)



from pandas import DataFrame
import pandas as pd

from dhlab.api.dhlab_api import document_corpus, get_metadata


class Corpus:
    def __init__(
            self,
            doctype=None,
            author=None,
            freetext=None,
            fulltext=None,
            from_year=None,
            to_year=None,
            from_timestamp=None,
            to_timestamp=None,
            title=None,
            ddk=None,
            subject=None,
            lang=None,
            limit=None):

        self.corpus = document_corpus(
            doctype,
            author,
            freetext,
            fulltext,
            from_year,
            to_year,
            from_timestamp,
            to_timestamp,
            title,
            ddk,
            subject,
            lang,
            limit
        )

        self.size = len(self.corpus)



    def add(self, corpus = None):
        """Add a corpus to existing corpus"""
        new_corpus = pd.concat([self.corpus, corpus.corpus]).drop_duplicates()
        self.corpus = new_corpus
        self.size = len(self.corpus)
        
    

class Corpus_from_identifiers(Corpus):
    def __init__(self, identifiers=None):
        self.corpus = get_metadata(urnlist(identifiers))


def urnlist(corpus):
    """Try to pull out a list of URNs from corpus"""

    if isinstance(corpus, Corpus):
        urnlist = list(corpus.corpus.urn)
    elif isinstance(corpus, DataFrame):
        urnlist = list(corpus.urn)
    elif isinstance(corpus, list):
        urnlist = corpus
    else:
        urnlist = []
    return urnlist



# legacy code
from dhlab.text import nbtokenizer
from dhlab.legacy import (
    nbtext,
    graph_networkx_louvain,
    token_map,
    nbpictures,
    nb_external_files,
    module_update
)

# code from further down in the code tree
from dhlab.text.corpus import Corpus
from dhlab.text.chunking import Chunks
from dhlab.text.conc_coll import Collocations, Concordance, Counts
from dhlab.text.geo_data import GeoData
from dhlab.ngram.ngram import Ngram, NgramBook, NgramNews

from dhlab.wordbank.wordbank import WordParadigm, WordLemma, WordForm


import pandas as pd
import requests

from dhlab.constants import BASE_URL


def get_metadata(urns=None):
    """ Fetch metadata from a list of urns """
    params = locals()
    r = requests.post(f"{BASE_URL}/get_metadata", json=params)
    return pd.DataFrame(r.json())



import pandas as pd
from dhlab.api.dhlab_api import (
    word_variant, 
    word_paradigm, 
    word_paradigm_many,
    word_form, 
    word_form_many, 
    word_lemma, 
    word_lemma_many
)

class WordParadigm():
    """Fetch inflection paradigms for a list of words, or just one word"""
    def __init__(self, words, lang='nob'):
        if isinstance(words, list):
            self.paradigms = pd.DataFrame(word_paradigm_many(words, lang))
        elif isinstance(words, str):
            self.paradigms = pd.DataFrame(word_paradigm(words, lang))
        else:
            self.paradigms = None
            
class WordForm():
    """Fetch possible forms of a word or list of words"""
    def __init__(self, words, lang='nob'):
        if isinstance(words, list):
            self.forms = pd.DataFrame(word_form_many(words, lang))
        elif isinstance(words, str):
            self.forms = pd.DataFrame(word_form(words, lang))
        else:
            self.forms = None

class WordLemma():
    """Fetch possbile lemmas for a given word form"""
    def __init__(self, words, lang='nob'):
        if isinstance(words, list):
            self.lemmas = pd.DataFrame(word_lemma_many(words, lang))
        elif isinstance(words, str):
            self.lemmas = pd.DataFrame(word_lemma(words, lang))
        else:
            self.lemmas = None
            
    
    



import re

import pandas as pd

from dhlab.api.dhlab_api import get_document_frequencies, concordance, urn_collocation
from dhlab.text.corpus import urnlist


# convert cell to a link
def make_link(row):
    r = "<a target='_blank' href = 'https://urn.nb.no/{x}'>{x}</a>".format(
        x=str(row))
    return r


# find hits a cell
def find_hits(x): return ' '.join(re.findall("<b>(.+?)</b", x))


class Concordance:
    """Wrapper for concordance function with added functionality"""

    def __init__(self, corpus=None, query=None, window=20, limit=500):

        self.concordance = concordance(urns=urnlist(corpus), words=query, window=window, limit=limit)
        self.concordance['link'] = self.concordance.urn.apply(make_link)
        self.concordance = self.concordance[['link', 'urn', 'conc']]
        self.concordance.columns = ['link', 'urn', 'concordance']
        self.corpus = corpus
        self.size = len(self.concordance)

    def show(self, n=10, style=True):
        if style:
            result = self.concordance.sample(min(n, self.size))[
                ['link', 'concordance']].style
        else:
            result = self.concordance.sample(min(n, self.size))
        return result


class Collocations():
    """Collocations """

    def __init__(
        self,
        corpus=None,
        words=None,
        before=10,
        after=10,
        reference=None,
        samplesize=20000
    ):
        if isinstance(words, str):
            words = [words]
        coll = pd.concat(
            [
                urn_collocation(
                    urns=urnlist(corpus),
                    word=w,
                    before=before,
                    after=after,
                    samplesize=samplesize
                )
                for w in words
            ]
        )[['counts']]

        self.coll = coll.groupby(coll.index).sum()
        self.reference = reference
        self.before = before
        self.after = after

        if reference is not None:
            teller = self.coll.counts / self.coll.counts.sum()
            divisor = self.reference.freq / self.reference.freq.sum()
            self.coll['relevance'] = teller / divisor

    def show(self, sortby='counts', n=20):
        return self.coll.sort_values(by=sortby, ascending=False)

    def keywordlist(self, top=200, counts=5, relevance=10):
        mask = self.coll[self.coll.counts > counts]
        mask = mask[mask.relevance > relevance]
        return list(mask.sort_values(
            by='counts', ascending=False).head(200).index)


class Counts():
    """Provide counts for a corpus - shouldn't be too large"""

    def __init__(self, corpus=None, words=None):
        if corpus is None and words is None:
            self.counts = None
        elif not corpus is None:
            # count - if words is none result will be as if counting all words
            # in the corpus
            self.counts = get_document_frequencies(
                urns=urnlist(corpus), cutoff=0, words=words)
